function [longmod,latmod,linmodel,TF]=linearize_UAV(OperatingPoint,verbose)
% [longmod,latmod,linmodel,TF]=linearize_UAV(OperatingPoint,verbose)
%
% Linearizes the UAV model about a given operating point using
% ../NL_Sim/UAV_NL.mdl. This function can be called from any of the three
% sim directories. However, this function will use your workspace
% variables. Requires the Control System Toolbox and Simulink Control
% Design.
%
% Inputs:
%   OperatingPoint - Operating point object of a trim condition
%   verbose        - boolean flag to suppress output; default "true"
%
% Outputs:
%   longmod  - longitudinal linear model
%   latmod   - lateral directional linear model
%   linmodel - full linear model
%   TF       - common input/output transfer functions
%
% Calls: eigpara.m
%
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
%
% SVN Info: $Id: linearize_UAV.m 305 2011-03-16 16:17:53Z murch $

% verbose output on by default
if nargin < 2
    verbose = 1;
end

%% Load model into memory
isLoaded = bdIsLoaded('UAV_NL'); % check to see if model is loaded or open
if ~isLoaded
    load_system('../NL_Sim/UAV_NL.mdl') % load model into system memory without opening diagram
end

%% LINEARIZE
linmodel = linearize('UAV_NL',OperatingPoint.op_point);

% Create Full 13-state Linear Model
set(linmodel, 'InputName', {'\delta_e' ;'\delta_a'; '\delta_r';'\delta_t';'\delta_f'});
set(linmodel, 'OutputName',{'V'; '\beta'; '\alpha'; 'h'; '\phi'; '\theta';'\psi';'p';'q';'r';'gamma'});
set(linmodel, 'StateName', {'\phi';'\theta';'\psi';'p';'q';'r';'u';'v';'w';'Xe';'Ye';'Ze';'\omega'})

% Generate transfer functions from statespace data
TF.full_elev2q = tf(linmodel(9,1));% elev to output 9 (q)
TF.full_ail2p  = tf(linmodel(8,2));% ail to output 8 (p)
TF.full_ail2r  = tf(linmodel(10,2));% ail to output 10 (r)
TF.full_rud2p  = tf(linmodel(8,3));% rud to output 8 (p)
TF.full_rud2r  = tf(linmodel(10,3));% rud to output 10 (r)

%% GENERATE LONGITUDINAL LINEAR MODEL
% Longitudinal dynamics
% States: u(7)  w(9)  q(5) theta(2) Ze(12) omega(13)
% Inputs: elevator(1) throttle(4)
% Outputs: Vs(1) alpha(3) q(9) theta(6) h (4)

% Generate State-space matrices for Longitudinal Model
% Indices for desired state, outputs, and inputs
Xlon = [7 9 5 2 12 13];
Ylon = [1 3 9 6 4];
Ilon = [1 4];
longmod = modred(linmodel(Ylon,Ilon),setdiff(1:13,Xlon),'Truncate');
longmod = xperm(longmod,[3 4 1 2 5 6]); % reorder state

% Generate transfer functions from statespace data
% TFs from elevator to outputs
TF.elev2Vs    = tf(longmod(1,1));% elev to output 1 (V)
TF.elev2alpha = tf(longmod(2,1));% elev to output 2 (alpha)
TF.elev2q     = tf(longmod(3,1));% elev to output 3 (q)
TF.elev2theta = tf(longmod(4,1));% elev to output 4 (theta)
TF.elev2h     = tf(longmod(5,1));% elev to output 5 (h)

% TFs from throttle to outputs
TF.thr2Vs    = tf(longmod(1,2));% thr to output 1 (V)
TF.thr2alpha = tf(longmod(2,2));% thr to output 2 (alpha)
TF.thr2q     = tf(longmod(3,2));% thr to output 3 (q)
TF.thr2theta = tf(longmod(4,2));% thr to output 4 (theta)
TF.thr2h     = tf(longmod(5,2));% thr to output 5 (h)

if verbose
    fprintf('\n');
    fprintf('\n Longitudinal Dynamics');
    fprintf('\n-----------------------');
    fprintf('\n  State vector: x = [u w q theta Ze Omega]');
    fprintf('\n  Input vector: u = [elevator throttle]');
    fprintf('\n Output vector: y = [Vs alpha q theta h]');
    fprintf('\n State matrix: A = \n\n\n');
    disp(longmod.A);
    fprintf('Longitudinal Poles:');
    damp(longmod.A)
    fprintf('\n\n\n Control matrix: B = \n');
    disp(longmod.B);
    fprintf('\n Observation matrix: C = \n');
    disp(longmod.C);
    
    % Eigenvalue analysis
    eiglon = eig(longmod.A);
    for i=1:length(eiglon);
        if imag(eiglon(i))>0
            [wd, T, wn, zeta] = eigpara(eiglon(i));
            fprintf('\n Eigenvalue: %6.4f +/- %6.4f i', real(eiglon(i)), imag(eiglon(i)));
            fprintf('\n Damping = %6.4f, natural frequency = %6.4f rad/s, period = %8.4f s', zeta, wn, T);
        elseif imag(eiglon(i))==0
            fprintf('\n Eigenvalue: %6.4f', eiglon(i));
            fprintf('\n Time constant = %6.4f s', -1/eiglon(i));
        end
    end
end
%% GENERATE LATERAL-DIRECTIONAL LINEAR MODEL
% Lateral-directional dynamics
% States: v(8) p(4) r(6) phi(1) psi(3)
% Inputs: aileron(2) rudder(3)
% Outputs: beta(2) p(8) r(10) phi(5) psi(7)

% Generate State-space matrices for lateral-directional Model
% Indices for desired state, outputs, and inputs
Xlat = [8 4 6 1 3];
Ylat = [2 8 10 5 7];
Ilat = [2 3];
latmod = modred(linmodel(Ylat,Ilat),setdiff(1:13,Xlat),'Truncate');
latmod = xperm(latmod,[5 3 4 1 2]); % reorder state

% Generate transfer functions from statespace data
% TFs from aileron to outputs
TF.ail2v   = tf(latmod(1,1));% ail to output 1 (v)
TF.ail2p   = tf(latmod(2,1));% ail to output 2 (p)
TF.ail2r   = tf(latmod(3,1));% ail to output 3 (r)
TF.ail2phi = tf(latmod(4,1));% ail to output 4 (phi)
TF.ail2psi = tf(latmod(5,1));% ail to output 5 (psi)

% TFs from rud to outputs
TF.rud2v   = tf(latmod(1,2));% rud to output 1 (v)
TF.rud2p   = tf(latmod(2,2));% rud to output 2 (p)
TF.rud2r   = tf(latmod(3,2));% rud to output 3 (r)
TF.rud2phi = tf(latmod(4,2));% rud to output 4 (phi)
TF.rud2psi = tf(latmod(5,2));% rud to output 5 (psi)

if verbose
    fprintf('\n');
    fprintf('\n Lateral-directional Dynamics');
    fprintf('\n------------------------------');
    fprintf('\n  State vector: x = [v p r phi psi]');
    fprintf('\n  Input vector: u = [aileron rudder]');
    fprintf('\n Output vector: y = [beta p r phi psi]');
    fprintf('\n State matrix: A = \n\n\n');
    disp(latmod.A);
    fprintf('Lateral-directional Poles:');
    damp(latmod.A)
    fprintf('\n\n\n Control matrix: B = \n');
    disp(latmod.B);
    fprintf('\n Observation matrix: C = \n');
    disp(latmod.C);
    
    % Eigenvalue analysis
    eiglat = eig(latmod.A);
    for i=1:length(eiglat)
        if imag(eiglat(i))>0
            [wd, T, wn, zeta] = eigpara(eiglat(i));
            fprintf('\n Eigenvalue: %6.4f +/- %6.4f i', real(eiglat(i)), imag(eiglat(i)));
            fprintf('\n Damping = %6.4f, natural frequency = %6.4f rad/s, period = %8.4f s', zeta, wn, T);
        elseif (imag(eiglat(i))==0)&&(real(eiglat(i))~=0)
            fprintf('\n Eigenvalue: %6.4f', eiglat(i));
            fprintf('\n Time constant = %6.4f s', -1/eiglat(i));
        end
    end
    fprintf('\n')
end

%% Cleanup
if ~isLoaded
    bdclose UAV_NL % clear model from system memory if we had to load it
end