%% UMN UAV Simulation: Linearize Tutorial
% This tutorial walks through the steps of linearizing the UMN UAV
% Simulation model. Most of these steps are handled in the "setup.m" and
% "linearize_UAV.m" functions provided with the UMN UAV sim. However, this
% tutorial will give you an in-depth understanding of how these functions
% work.

%%
%  Author: Austin Murch
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
% SVN Info: $Id: linearize_tutorial.m 305 2011-03-16 16:17:53Z murch $

%% Setup and Configuration
% Before we get started we need to make sure the MATLAB path is correct and
% we have all of the aircraft and simulation parameters are defined. This
% step is handled in setup.m.

% Add Libraries folder to MATLAB path
addpath ../Libraries
warning('off','Simulink:SL_LoadMdlParameterizedLink')

% Configure Airframe, either 'Ultrastick' or 'FASER'
[AC,Env] = UAV_config('Ultrastick');

% Simulation sample time
SampleTime = 0.02; % sec

% Load model into system memory without opening diagram
load_system('UAV_NL.mdl') 

%% Set Aircraft Initial Conditions and Trim the Model
% Next we need to define initial conditions for the simulation model inputs
% and state vector and trim the model to a particular flight condition.
% This step is handled in setup.m.

% Set the initial model inputs
TrimCondition.Inputs.elevator = 0.091; % rad
TrimCondition.Inputs.aileron  = 0;     % rad
TrimCondition.Inputs.rudder   = 0;     % rad
TrimCondition.Inputs.flap     = 0;     % rad
TrimCondition.Inputs.throttle = 0.559; % nd, 0 to 1

% Set initial state values
TrimCondition.InertialIni    = [0 0 -100]';   % Initial Position in Inertial Frame [Xe Ye Ze], [m]
TrimCondition.LLIni          = [45 -122];     % Initial Latitude/Longitude of Aircraft [Lat Long], [deg]
TrimCondition.VelocitiesIni  = [17 0 0.369]'; % Initial Body Frame velocities [u v w], [m/s]
TrimCondition.AttitudeIni    = [0 0.0217 0]'; % Initial Euler orientation [roll,pitch,yaw] [rad]
TrimCondition.RatesIni       = [0 0 0]';      % Initial Body Frame rotation rates [p q r], [rad/s]
TrimCondition.EngineSpeedIni = 827;           % Initial Engine Speed [rad/s]

% Set the trim target
TrimCondition.target = struct('V_s',17,'gamma',0); % straight and level, (m/s, rad)

% Find the trim solution
[TrimCondition,OperatingPoint] = trim_UAV(TrimCondition,AC);

%% Linearize the Model
% Linearizing is done using the "linearize.m" built-in function. We need to
% pass this function the model name and an operating point object that we
% get from trim_UAV. A linear model is only relevant when created around a
% trimmed flight condition.

linmodel = linearize('UAV_NL',OperatingPoint.op_point);

%% Full Linear Model
% We now have a full 13-state linear model, but the state, input, and
% output names need to be made consistent. Note the "\" in the name will
% format the name as a greek symbol when plotting.
set(linmodel, 'InputName', {'\delta_e' ;'\delta_a'; '\delta_r';'\delta_t';'\delta_f'});
set(linmodel, 'OutputName',{'V'; '\beta'; '\alpha'; 'h'; '\phi'; '\theta';'\psi';'p';'q';'r';'gamma'});
set(linmodel, 'StateName', {'\phi';'\theta';'\psi';'p';'q';'r';'u';'v';'w';'Xe';'Ye';'Ze';'\omega'})
linmodel

%% 
% From the full model we can generate transfer functions between the inputs
% and the relevant outputs. These are stored in the "TF" data structure.

% Generate transfer functions from statespace data
TF.full_elev2q = tf(linmodel(9,1));% elev to output 9 (q)
TF.full_ail2p  = tf(linmodel(8,2));% ail to output 8 (p)
TF.full_ail2r  = tf(linmodel(10,2));% ail to output 10 (r)
TF.full_rud2p  = tf(linmodel(8,3));% rud to output 8 (p)
TF.full_rud2r  = tf(linmodel(10,3));% rud to output 10 (r)

%% Reduced Order Longitudinal Model
% The full 13 state linear model is a bit cumbersome to work with, so we
% normally separate the model into two decoupled, reduced order models: one
% for the longitudinal axis and another for the lateral-directional axes.
%
% The longitudinal model has 5 states: forward velocity, vertical velocity, 
% pitch rate, pitch angle, vertical position, and motor speed (u, w, q,
% \theta, Ze, \omega). 2 inputs: elevator and throttle, (\delta_e,
% \delta_t). and 5 outputs: airspeed, angle of attack, pitch rate, pitch
% angle, and altitude. (V_s, \alpha, q, \theta, h). We can reduce the full
% linear model using model reduction with the "modred" function. We first
% define indices corresponding to the states, outputs, and inputs we wish
% to retain in the linear model. We then need to reorder the state vector
% to the order listed above.

% Generate State-space matrices for Longitudinal Model
% Indices for desired state, outputs, and inputs that we want to keep
Xlon = [7 9 5 2 12 13];
Ylon = [1 3 9 6 4];
Ilon = [1 4];
longmod = modred(linmodel(Ylon,Ilon),setdiff(1:13,Xlon),'Truncate');

% Reorder state vector
longmod = xperm(longmod,[3 4 1 2 5 6])

%%
% Now we have a reduced order linear model for the longitudinal dynamics.
% From this linear model we can generate transfer functions between the
% inputs and the relevant outputs. These are stored in the "TF" data
% structure.

% Generate transfer functions from statespace data
% TFs from elevator to outputs
TF.elev2Vs    = tf(longmod(1,1));% elev to output 1 (V)
TF.elev2alpha = tf(longmod(2,1));% elev to output 2 (alpha)
TF.elev2q     = tf(longmod(3,1));% elev to output 3 (q)
TF.elev2theta = tf(longmod(4,1));% elev to output 4 (theta)
TF.elev2h     = tf(longmod(5,1));% elev to output 5 (h)

% TFs from throttle to outputs
TF.thr2Vs    = tf(longmod(1,2));% thr to output 1 (V)
TF.thr2alpha = tf(longmod(2,2));% thr to output 2 (alpha)
TF.thr2q     = tf(longmod(3,2));% thr to output 3 (q)
TF.thr2theta = tf(longmod(4,2));% thr to output 4 (theta)
TF.thr2h     = tf(longmod(5,2));% thr to output 5 (h)

%% Reduced Order Lateral-Directional Model
% The lateral-directional model has 5 states: side velocity,  
% roll rate, yaw rate, roll angle, and yaw angle . (v, p, r, \phi, \psi). 2
% inputs: aileron and rudder, (\delta_a, \delta_r). and 5 outputs: sideslip
% angle, roll rate, yaw rate, roll angle, and yaw angle (\beta, p, r, \phi,
% \psi). We can reduce the full linear model using model reduction with the
% "modred" function. We first define indices corresponding to the states,
% outputs, and inputs we wish to retain in the linear model. We then need
% to reorder the state vector to the order listed above.

% Generate State-space matrices for lateral-directional Model
% Indices for desired state, outputs, and inputs that we want to keep
Xlat = [8 4 6 1 3];
Ylat = [2 8 10 5 7];
Ilat = [2 3];
latmod = modred(linmodel(Ylat,Ilat),setdiff(1:13,Xlat),'Truncate');
latmod = xperm(latmod,[5 3 4 1 2]); % reorder state

%%
% Now we have a reduced order linear model for the lateral-directional
% dynamics. From this linear model we can generate transfer functions
% between the inputs and the relevant outputs. These are stored in the "TF"
% data structure.

% Generate transfer functions from statespace data
% TFs from aileron to outputs
TF.ail2v   = tf(latmod(1,1));% ail to output 1 (v)
TF.ail2p   = tf(latmod(2,1));% ail to output 2 (p)
TF.ail2r   = tf(latmod(3,1));% ail to output 3 (r)
TF.ail2phi = tf(latmod(4,1));% ail to output 4 (phi)
TF.ail2psi = tf(latmod(5,1));% ail to output 5 (psi)

% TFs from rud to outputs
TF.rud2v   = tf(latmod(1,2));% rud to output 1 (v)
TF.rud2p   = tf(latmod(2,2));% rud to output 2 (p)
TF.rud2r   = tf(latmod(3,2));% rud to output 3 (r)
TF.rud2phi = tf(latmod(4,2));% rud to output 4 (phi)
TF.rud2psi = tf(latmod(5,2));% rud to output 5 (psi)

%% Longitudinal Model Analysis
% Now that we have linear models, let's look at the bode plot of a few
% relevant cases: first we'll look at elevator to pitch rate, comparing the
% full model to the reduced order model.

bode(TF.full_elev2q,TF.elev2q,{.01,1000}); grid on
legend('Full Model','Longitudinal')

%%
% We see that there is very little difference between the two models for
% any reasonable frequencies, so the reduced order model is a good
% approximation. We can also see the two dominant modes of the system, the
% Phugoid and short period, in the two humps in the magnitude plot. Also of
% interest is where the magnitude crosses zero- this defines the bandwidth
% of the system response.
%
% The best way to look at the modes of the system is to look at the
% Eigenvalues; for this, we'll use the "damp" function.

damp(longmod)

%%
% This shows us the Eigenvalues of the system, as well as their frequency
% and damping. We see the Phugoid mode, with reasonable damping and normal
% low frequency. Note that the short period doesn't show up as a complex
% pair of Eigenvalues as we would normally expect. This is because with the
% current aerodynamic model, the short period mode is critically damped,
% and thus non-oscillatory.

%% Lateral-Directional Model Analysis
% For the lateral-directional model, let's look at aileron to roll rate and
% rudder to yaw rate.

bode(TF.full_ail2p,TF.ail2p,{.01,1000}); grid on
legend('Full Model','Longitudinal')

%%
% Again, we see that there is very little difference between the two models
% for any reasonable frequencies, so the reduced order model is a good
% approximation. We can also see the dominant behavior of the roll axis, in
% that the roll rate response behaves as a first-order system.
%
% Now the rudder to yaw rate:

bode(TF.full_rud2r,TF.rud2r,{.01,1000}); grid on
legend('Full Model','Longitudinal')

%%
% Again, the reduced order model is a good approximation. The hump in the
% magnitude is the Dutch Roll mode.
%
% The best way to look at the modes of the system is to look at the
% Eigenvalues; for this, we'll use the "damp" function.

damp(latmod)

%%
% This shows us the Eigenvalues of the system, as well as their frequency
% and damping. We see the Dutch Roll mode is the only complex pair of
% Eigenvalues, with a good damping ratio and moderate frequency. The roll
% mode is first order and relatively fast; the slow first order mode is the
% spiral mode, which can be unstable for some aircraft.
%
% This concludes the linearize tutorial. Please send any comments or
% questions to Austin Murch (murch@aem.umn.edu).