/******************************************************************************
 * FILE: airDataSys.c
 *
 * DESCRIPTION:
 * C class header: Microbotics: SPA20422 AIR DATA SYSTEM
 *
 * CREATED: 09-28-10: David Escobar Sanabria
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: airDataSys.c 253 2011-01-20 22:01:35Z murch $
 ******************************************************************************/

#include "airDataSys.h"

/// Initialization of pressure sensor suite
int airDataSysStart(char* serial_port, int baudrate) {

    ads_buff_len = sizeof (ads_buff_info);
    if (cyg_io_lookup(serial_port, &ads_console_handle) < 0) {
        fprintf(stderr, "\n <IMUDAQ thread>: unable to obtain handle for %s - %s\n", serial_port, strerror(errno));
    }

    airDataSysPortID = initSerial(serial_port, baudrate);

    if (airDataSysPortID == -1) {
        fprintf(stderr, "\n Error: Can not open serial port \n");
    } else {
        setADSBinary();
        cyg_thread_delay(200);

    }
    fprintf(stderr, "\n Air data system started  \n");
    return airDataSysPortID;
}


/// Initialization of serial port
int initSerial(char *portname, int baudrate) {
    int port_hdl;
    /// open the serial port used for gps communication
    port_hdl = open_serial(portname, baudrate);
    if (port_hdl == -1) {
        fprintf(stderr, "Open serial port error!\n");
        return -1;
    }
    cyg_thread_delay(100);
    return port_hdl;
}

/// Obtain sensor data frame - array of unsigned 8 bit integers (uint8_t)
int getAirDataSysFrame(uint8_t *dataPacket) {

    counterFrameAttemps = 0;
    counterFrame = 0;
    cs0 = 0;
    cs1 = 0;

    cyg_io_get_config(ads_console_handle, CYG_IO_GET_CONFIG_SERIAL_BUFFER_INFO, &ads_buff_info, &ads_buff_len);

    /// Check if there are bytes in the buffer
    adsBytesInBuffer = ads_buff_info.rx_count;
    if (adsBytesInBuffer > 0) {
        while (end == 0) {

            /// The sensor protocol and the data are not matching.
            if (counterFrameAttemps >= MAX_FRAME_ATTEMPS) {
                //fprintf(stderr, "Attemps Frame:  %d \n", counterFrameAttemps);
                return ADS_ERROR_PROTOCOL;
            }

            if (counterFrame >= AIR_DATA_SYS_FRAME_SIZE) {
                counterFrame = 0;
                /// Frame has the correct header
                if (bufferIn[0] == 129 && bufferIn[1] == 161) {

                    /// Checksum is succesfull
                    if (cs0 == bufferIn[26] && cs1 == bufferIn[27]) {
                        /// Creates a copy of the read buffer
                        /// dataPacket points to this new memory location
                        memcpy(dataPacket, bufferIn, sizeof (bufferIn));
                        return 0;
                    }                        /// Checksum failed
                    else {
                        return ADS_ERROR_CHECKSUM;
                    }

                }
                    /// The bytes dont correspond to the frame we are looking for.
                else {
                    counterFrameAttemps++;
                    //fprintf(stderr,  "lost frames %d  "  ,counterFrameAttemps );
                }

                cs0 = 0;
                cs1 = 0;

            }

            /// Reading of bytes:
            counterByteAttemps = 0;
            while (len == 0) {

                len = read(airDataSysPortID, byteIn, 1);
                counterByteAttemps++;
                if (counterByteAttemps >= MAX_BYTE_ATTEMPS) {
                    //fprintf(stderr, "Attemp END:  %d \n", counterByteAttemps);
                    return ADS_ERROR_NO_DATA;
                }
                if (len == -1) {
                    //fprintf(stderr, "\n Reading error!!!");
                }
            }
            //fprintf(stderr, "Attemp END:  %d \n", counterByteAttemps);
            len = 0;
            bufferIn[counterFrame] = byteIn[0];

            if (counterFrame > 1 && bufferIn[0] == 129 && bufferIn[1] == 161) {

                /// Verify end of frame (before CS0 and CS1)
                if (counterFrame < 26) {
                    /// Fletcher checksum
                    cs0 = cs0 + bufferIn[counterFrame];

                    if (cs0 >= 256) {
                        cs0 = cs0 - 256;
                    }
                    cs1 = cs1 + cs0;

                    if (cs1 >= 256) {
                        cs1 = cs1 - 256;
                    }
                }

            }
            counterFrame++;
            /// Frame synchronization:

            if (bufferIn[0] != 129) {
                counterFrame = 0;
            }


        } //end while
    }

    return 0;
}

int getADSStatus(uint8_t *dataFrame) {
    return (uint16_t) ((dataFrame[4] << 8) | dataFrame[5]);
}

int getADSUTime(uint8_t *dataFrame) {
    return (uint16_t) ((dataFrame[6] << 8) | dataFrame[7]);
}

double getADSAbsolutePressure(uint8_t *dataFrame) {
    return (double)( ( (double)( (uint16_t) ((dataFrame[8] << 8) | dataFrame[9]) ) ) / 100 );
}

double getADSEffectiveSeaLevelPressure(uint8_t *dataFrame) {
    return (double) ( ((double) ( (uint16_t) ((dataFrame[10] << 8) | dataFrame[11]) ) ) / 100);
}

double getADSAltitude(uint8_t *dataFrame) {
    return (double) ( ( (double)( (int32_t) (((int8_t) dataFrame[12] << 24) | (dataFrame[13] << 16) | (dataFrame[14] << 8) | (dataFrame[15])) ) ) / 10 );
}

double getADSInternalTemperature(uint8_t *dataFrame) {
    return  (double) (  ( (double)( (int16_t) (((int8_t) dataFrame[16] << 8) | dataFrame[17])  ) ) / 10 );
}

double getADSExternalTemperature(uint8_t *dataFrame) {
    return (double) ( ( (double)( (int16_t) (((int8_t) dataFrame[18] << 8) | dataFrame[19]) ) )/10 );
}

int getADSDryAirDensity(uint8_t *dataFrame) {
    return (uint16_t) ((dataFrame[20] << 8) | dataFrame[21]);
}

double getADSDifferentialPressure(uint8_t *dataFrame) {
    return  (double) ( ( (double)( (int16_t) (((int8_t) dataFrame[22] << 8) | dataFrame[23]) ) )  /1000);
}

double getADSAirSpeed(uint8_t *dataFrame) {
    // Given in KPH * 10
	return (double) ( ( (double) ((uint16_t) ((dataFrame[24] << 8) | dataFrame[25]) ) ) / 36 );
}

void updateADSAltitude(int altitude) {
    uint8_t bufferOut[10];

    sprintf(bufferOut, "~h%d\n", altitude);
    //fprintf(stderr, "size: %d", strlen(bufferOut) );
    write(airDataSysPortID, bufferOut, strlen(bufferOut));

}

void updateADSSeaPressure(int P0) {
    uint8_t bufferOut[10];

    sprintf(bufferOut, "~r%d\n", P0);
    //fprintf(stderr, "size: %d", strlen(bufferOut) );
    write(airDataSysPortID, bufferOut, strlen(bufferOut));

}

void resetADSDifferentialPressure() {
    write(airDataSysPortID, "~v\n", 3);
}

void setADSFrameRate(int n50ms) {
    uint8_t bufferOut[10];
    sprintf(bufferOut, "~m%d\n", n50ms);
    //fprintf(stderr, "size: %d", strlen(bufferOut) );
    write(airDataSysPortID, bufferOut, strlen(bufferOut));
}

void setADSBinary() {
    write(airDataSysPortID, "~b\n", 3);
}
