/*
 * FILE: gps_crescent.c
 * DESCRIPTION:
 * Driver for decoding binary GPS message from CRESCENT GPS receiver
 *
 *  Created on: Feb 27, 2010
 *      Author: Zhefeng Li, Curt Olson, Rohit Pandita
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: gps_crescent.c 253 2011-01-20 22:01:35Z murch $
 */

#include <termios.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>

#include "gps_crescent.h"
#include "../../utils/serial_mpc5200.h"


// swap big/little endian bytes
void my_swap( byte *buf, int index, int count ) {
	int i;
	byte tmp;
	for ( i = 0; i < count / 2; ++i ) {
		tmp = buf[index+i];
		buf[index+i] = buf[index+count-i-1];
		buf[index+count-i-1] = tmp;
	}
}

// calculate the nmea check sum
unsigned char calc_nmea_cksum(byte *sentence) {
	byte sum = 0;
	int i, len;

	len = strlen(sentence);
	sum = sentence[0];
	for ( i = 1; i < len; i++ ) {
		// cout << sentence[i];
		sum ^= sentence[i];
	}

	// printf("sum = %02x\n", sum);
	return sum;
}


int parse_gps( int blockid, byte *buf, int datalen, struct gps *gpspkt )
{
	if ( blockid == 1 ) {

		my_swap(buf, 2, 2);
		my_swap(buf, 4, 8);
		my_swap(buf, 12, 8);
		my_swap(buf, 20, 8);
		my_swap(buf, 28, 4);
		my_swap(buf, 32, 4);
		my_swap(buf, 36, 4);
		my_swap(buf, 40, 4);
		my_swap(buf, 44, 4);
		my_swap(buf, 48, 2);
		my_swap(buf, 50, 2);

		byte *p = buf;
		byte num_sats = p[1];
		cyg_uint16 gps_week = *((cyg_uint16 *)(p+2));
		double ITOW = *((double *)(p+4));

		gpspkt->GPS_TOW = (unsigned int)ITOW;
		gpspkt->lat = *((double *)(p+12));
		gpspkt->lon = *((double *)(p+20));
		gpspkt->alt = *((double *)(p+28));
		gpspkt->vn = *((double *)(p+32));
		gpspkt->ve = *((double *)(p+36));
		gpspkt->vd = *((double *)(p+40)) * -1;

		cyg_uint16 nav_mode = *((cyg_uint16 *)(p+48));

		if ( nav_mode == 0 || num_sats > 18 )
			gpspkt->err_type = gps_nolock;
		else
			gpspkt->err_type = data_valid;
	}

	return 0;
}


// Initialize GPS and configure it to send the desired messages
int init_gps(char *portname, int baudrate)
{
	int port_hdl;
	// open the serial port used for gps communication
	port_hdl = open_serial( portname, baudrate );
	if ( port_hdl == -1 )	{
		fprintf( stderr, "Open serial port error!\n" );
		return -1;
	}
	cyg_thread_delay(100);

	byte buf[100];
	int len;

	// configure the gps to send the message we want
	strcpy(buf, "$JBIN,1,1");
	buf[9] = 0x0D;
	buf[10] = 0x0A;
	len = write( port_hdl, buf, 11 );
	//	fprintf(stderr, "wrote %d bytes (should be 11)\n", len);
	cyg_thread_delay(100);

	strcpy(buf, "$JBIN,96,0");
	buf[10] = 0x0D;
	buf[11] = 0x0A;
	len = write( port_hdl, buf, 12 );
	//	fprintf(stderr, "wrote %d bytes (should be 12)\n", len);
	cyg_thread_delay(100);

return port_hdl;
}


int read_gps(struct gps *gpspkt, int port)
{
	static int state = 0;
	static int blockid = 0, block_lo = 0, block_hi = 0;
	static int datalen = 0, len_lo = 0, len_hi = 0;
	static int counter = 0;
	static cyg_uint16 cksum = 0, cksum_lo = 0, cksum_hi = 0;
	int len;
	int i;
	byte input[500];
	byte message[500];

	if ( state == 0 ) {
		counter = 0;
		cksum = 0;
		len = read( port, input, 1 );
		while ( len > 0 && input[0] != '$' ) {
			// fprintf( stderr, "state0: len = %d val = %2X\n", len, input[0] );
			len = read( port, input, 1 );
		}
		if ( len > 0 && input[0] == '$' ) {
			// fprintf( stderr, "read $\n");
			state++;
		}
	}
	if ( state == 1 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			if ( input[0] == 'B' ) {
				// fprintf( stderr, "read B\n");
				state++;
			} else {
				state = 0;
			}
		}
	}
	if ( state == 2 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			if ( input[0] == 'I' ) {
				// fprintf( stderr, "read I\n");
				state++;
			} else {
				state = 0;
			}
		}
	}
	if ( state == 3 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			if ( input[0] == 'N' ) {
				// fprintf( stderr, "read N\n");
				state++;
			} else {
				state = 0;
			}
		}
	}
	if ( state == 4 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			block_lo = input[0];
			state++;
		}
	}
	if ( state == 5 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			block_hi = input[0];
			blockid = block_hi*256 + block_lo;
			// fprintf( stderr, "block id = %d\n", blockid );
			state++;
		}
	}
	if ( state == 6 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			len_lo = input[0];
			state++;
		}
	}
	if ( state == 7 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			len_hi = input[0];
			datalen = len_hi*256 + len_lo;
			// fprintf( stderr, "data len = %d\nRW: ", datalen );
			state++;
		}
	}
	if ( state == 8 ) {
		len = read( port, input, 1 );
		while ( len > 0 ) {
			message[counter++] = input[0];
			//fprintf( stderr, "%02X ", input[0] );
			cksum += input[0];
			if ( counter >= datalen ) {
				break;
			}
			len = read( port, input, 1 );
		}

		if ( counter >= datalen ) {
			state++;
			//fprintf( stderr, "\n" );
		}
	}
	if ( state == 9 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			cksum_lo = input[0];
			state++;
		}
	}
	if ( state == 10 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			cksum_hi = input[0];
			if ( cksum == cksum_hi*256 + cksum_lo ) {
				// fprintf( stderr, "checksum passes (%d)!\n", blockid );
				state++;
			} else {
				//				fprintf( stderr, \
				"checksum failed %d (computed) != %d (message)\n", \
				cksum, cksum_hi*256 + cksum_lo );
				state = 0;
			}
		}
	}
	if ( state == 11 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			if ( input[0] == 0x0D ) {
				// fprintf( stderr, "read CR\n");
				state++;
			} else {
				state = 0;
			}
		}
	}
	if ( state == 12 ) {
		len = read( port, input, 1 );
		if ( len > 0 ) {
			state = 0;		/* no matter, this is the end, reset
				   state to 0 */
			if ( input[0] == 0x0A ) {
				// fprintf( stderr, "read LF\n");
				parse_gps( blockid, message, datalen, gpspkt );
				return 1;	/* signal a successful read! */
			}
		}
	}

	return 0;
}

